@@ -234,16 +234,6 @@ class DensityFlow(nn.Module):
                     allow_broadcast=self.allow_broadcast,
                     use_cuda=self.use_cuda,
                 )
-        if self.loss_func == 'negbinomial':
-            self.decoder_total_count = MLP(
-                    [self.latent_dim] + self.decoder_hidden_layers + [self.input_size],
-                    activation=activate_fct,
-                    output_activation=Exp,
-                    post_layer_fct=post_layer_fct,
-                    post_act_fct=post_act_fct,
-                    allow_broadcast=self.allow_broadcast,
-                    use_cuda=self.use_cuda,
-                )
 
         if self.latent_dist == 'studentt':
             self.codebook = MLP(
@@ -324,9 +314,9 @@ class DensityFlow(nn.Module):
         batch_size = xs.size(0)
         self.options = dict(dtype=xs.dtype, device=xs.device)
         
-        #if self.loss_func=='negbinomial':
-        #    total_count = pyro.param("inverse_dispersion", self.inverse_dispersion *
-        #                             xs.new_ones(self.input_size), constraint=constraints.positive)
+        if self.loss_func=='negbinomial':
+            total_count = pyro.param("inverse_dispersion", self.inverse_dispersion *
+                                     xs.new_ones(self.input_size), constraint=constraints.positive)
             
         if self.use_zeroinflate:
             gate_logits = pyro.param("dropout_rate", xs.new_zeros(self.input_size))
@@ -370,7 +360,6 @@ class DensityFlow(nn.Module):
                     rate = theta * torch.sum(xs, dim=1, keepdim=True)
 
             if self.loss_func == 'negbinomial':
-                total_count = self.decoder_total_count(zs)
                 if self.use_zeroinflate:
                     pyro.sample('x', dist.ZeroInflatedDistribution(dist.NegativeBinomial(total_count=total_count, probs=theta),gate_logits=gate_logits).to_event(1), obs=xs)
                 else:
@@ -404,9 +393,9 @@ class DensityFlow(nn.Module):
         batch_size = xs.size(0)
         self.options = dict(dtype=xs.dtype, device=xs.device)
         
-        #if self.loss_func=='negbinomial':
-        #    total_count = pyro.param("inverse_dispersion", self.inverse_dispersion *
-        #                             xs.new_ones(self.input_size), constraint=constraints.positive)
+        if self.loss_func=='negbinomial':
+            total_count = pyro.param("inverse_dispersion", self.inverse_dispersion *
+                                     xs.new_ones(self.input_size), constraint=constraints.positive)
             
         if self.use_zeroinflate:
             gate_logits = pyro.param("dropout_rate", xs.new_zeros(self.input_size))
@@ -455,7 +444,6 @@ class DensityFlow(nn.Module):
                     rate = theta * torch.sum(xs, dim=1, keepdim=True)
 
             if self.loss_func == 'negbinomial':
-                total_count = self.decoder_total_count(zs)
                 if self.use_zeroinflate:
                     pyro.sample('x', dist.ZeroInflatedDistribution(dist.NegativeBinomial(total_count=total_count, probs=theta),gate_logits=gate_logits).to_event(1), obs=xs)
                 else:
@@ -489,9 +477,9 @@ class DensityFlow(nn.Module):
         batch_size = xs.size(0)
         self.options = dict(dtype=xs.dtype, device=xs.device)
         
-        #if self.loss_func=='negbinomial':
-        #    total_count = pyro.param("inverse_dispersion", self.inverse_dispersion *
-        #                             xs.new_ones(self.input_size), constraint=constraints.positive)
+        if self.loss_func=='negbinomial':
+            total_count = pyro.param("inverse_dispersion", self.inverse_dispersion *
+                                     xs.new_ones(self.input_size), constraint=constraints.positive)
             
         if self.use_zeroinflate:
             gate_logits = pyro.param("dropout_rate", xs.new_zeros(self.input_size))
@@ -552,7 +540,6 @@ class DensityFlow(nn.Module):
                     rate = theta * torch.sum(xs, dim=1, keepdim=True)
 
             if self.loss_func == 'negbinomial':
-                total_count = self.decoder_total_count(zs)
                 if self.use_zeroinflate:
                     pyro.sample('x', dist.ZeroInflatedDistribution(dist.NegativeBinomial(total_count=total_count, probs=theta),gate_logits=gate_logits).to_event(1), obs=xs)
                 else:
@@ -586,9 +573,9 @@ class DensityFlow(nn.Module):
         batch_size = xs.size(0)
         self.options = dict(dtype=xs.dtype, device=xs.device)
         
-        #if self.loss_func=='negbinomial':
-        #    total_count = pyro.param("inverse_dispersion", self.inverse_dispersion *
-        #                             xs.new_ones(self.input_size), constraint=constraints.positive)
+        if self.loss_func=='negbinomial':
+            total_count = pyro.param("inverse_dispersion", self.inverse_dispersion *
+                                     xs.new_ones(self.input_size), constraint=constraints.positive)
             
         if self.use_zeroinflate:
             gate_logits = pyro.param("dropout_rate", xs.new_zeros(self.input_size))
@@ -659,7 +646,6 @@ class DensityFlow(nn.Module):
                     rate = theta * torch.sum(xs, dim=1, keepdim=True)
 
             if self.loss_func == 'negbinomial':
-                total_count = self.decoder_total_count(zs)
                 if self.use_zeroinflate:
                     pyro.sample('x', dist.ZeroInflatedDistribution(dist.NegativeBinomial(total_count=total_count, probs=theta),gate_logits=gate_logits).to_event(1), obs=xs)
                 else:
@@ -856,35 +842,36 @@ class DensityFlow(nn.Module):
         
         return counts, zs
     
-    def _cell_response(self, xs, factor_idx, perturb):
+    def _cell_response(self, zs_basal, perturb_idx, perturb):
         #zns,_ = self.encoder_zn(xs)    
-        zns,_ = self._get_basal_embedding(xs)   
+        #zns,_ = self._get_basal_embedding(xs)   
+        zns = zs_basal
         if perturb.ndim==2:
-            ms = self.cell_factor_effect[factor_idx]([zns, perturb])
+            ms = self.cell_factor_effect[perturb_idx]([zns, perturb])
         else:
-            ms = self.cell_factor_effect[factor_idx]([zns, perturb.reshape(-1,1)])
+            ms = self.cell_factor_effect[perturb_idx]([zns, perturb.reshape(-1,1)])
             
         return ms 
 
     def get_cell_response(self, 
-                             xs, 
-                             factor_idx,
-                             perturb,
+                             zs_basal, 
+                             perturb_idx,
+                             perturb_us,
                              batch_size: int = 1024):
         """
         Return cells' changes in the latent space induced by specific perturbation of a factor
 
         """
-        xs = self.preprocess(xs)
-        xs = convert_to_tensor(xs, device=self.get_device())
-        ps = convert_to_tensor(perturb, device=self.get_device())
-        dataset = CustomDataset2(xs,ps)
+        #xs = self.preprocess(xs)
+        zs_basal = convert_to_tensor(zs_basal, device=self.get_device())
+        ps = convert_to_tensor(perturb_us, device=self.get_device())
+        dataset = CustomDataset2(zs_basal,ps)
         dataloader = DataLoader(dataset, batch_size=batch_size, shuffle=False)
 
         Z = []
         with tqdm(total=len(dataloader), desc='', unit='batch') as pbar:
-            for X_batch, P_batch, _ in dataloader:
-                zns = self._cell_response(X_batch, factor_idx, P_batch)
+            for Z_batch, P_batch, _ in dataloader:
+                zns = self._cell_response(Z_batch, perturb_idx, P_batch)
                 Z.append(tensor_to_numpy(zns))
                 pbar.update(1)
 
