@@ -1,12 +1,12 @@
 from .SURE import SURE
-from .PerturbFlow import PerturbFlow
+from .DensityFlow import DensityFlow
 
 from . import utils 
 from . import codebook
 from . import SURE
-from . import PerturbFlow
+from . import DensityFlow
 from . import atac
 from . import flow 
 from . import perturb
 
-__all__ = ['SURE', 'PerturbFlow', 'flow', 'perturb', 'atac', 'utils', 'codebook']
\ No newline at end of file
+__all__ = ['SURE', 'DensityFlow', 'flow', 'perturb', 'atac', 'utils', 'codebook']
\ No newline at end of file
